## Distributed as part of the supplementary material for the the paper
## ``Improving the accuracy of likelihood-based inference in
## meta-analysis and meta-regression''
##
## Authors: Ioannis Kosmidis, Annamaria Guolo, Cristiano Varin
## Date: 15 October 2016
## Licence: GPL 2 or greater

## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

## Simulate meta-analyses data from individual participant
## data meta-anlysis models, based on the simulation setting in
## Appendix B of
## ABO-ZAID, G., GUO, B., DEEKS, J. J., DEBRAY, T. P., STEYERBERG,
## E. W., MOONS, K. G. & RILEY, R. D. (2013). Individual participant
## data meta-analyses should not ignore clustering. J. Clin. Epidemiol. 66, 865–73.

## Arguments:
## num.studies  number of studies
## min.size     minimum number of subjects per study
## max.size     maximum number of subjects per study
## binary.x     should a Bernoulli be used to generate x? If FALSE then a Normal is used
## prevelance   if binary.x = TRUE, the probability of 1 when simulating x (assumed the same across studies)
## mean.x       if binary.x = FALSE, the mean of the Normal distribution for simulating x (assumed the same across studies)
## var.x        if binary.x = FALSE, the variance of the Normal distribution for simulating x
##              (assumed the same across studies)
## alpha        the 'true' baseline log-odds
## beta         the 'true' effect of x
## var.re       variance of the random effect
## br           if TRUE then brglm is used to estimate beta

simulate.IPDMA <- function(num.studies = 5,
                           min.size = 30,
                           max.size = 100,
                           binary.x = TRUE,
                           prevelance = 0.5,
                           mean.x = 4,
                           var.x = 1.5^2,
                           alpha = -1.27,
                           beta = 0,
                           var.a = 0.25^2,
                           var.b = 0.1^2,
                           br = FALSE) {
    if (br) require(brglm)
    ## Generate study sizes
    sizes <- sample(min.size : max.size, num.studies, replace = TRUE)

    betahat <- betahat.se <- rep(NA, num.studies)

    for (i in 1:num.studies) {
        ## Generate covariates for study i
        xi <- if (binary.x) rbinom(sizes[i], 1, prevelance) else rnorm(sizes[i], mean.x, sqrt(var.x))
        ## Generate responses for study i
        yi <- rbinom(sizes[i], 1, plogis(alpha + (beta + rnorm(1) * sqrt(var.b))*xi + rnorm(1) * sqrt(var.a)))
        dati <- data.frame(x = xi, y = yi)
        ## Estimate the logistic regression model for study i
        glmest <- if (br) brglm else glm
        mod <- glmest(y ~ x, data = dati, family = binomial("logit"))
        betahat[i] <- coef(mod)["x"]
        betahat.se[i] <- coefficients(summary(mod))["x", "Std. Error"]
    }
    list(betahat = betahat, betahat.se = betahat.se)
}


